@@ -1,6 +1,6 @@
 /* ncftp.h
  *
- * Copyright (c) 1996-2004 Mike Gleason, NcFTP Software.
+ * Copyright (c) 1996-2010 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -13,24 +13,33 @@ extern "C"
 {
 #endif	/* __cplusplus */
 
-#define kLibraryVersion "@(#) LibNcFTP 3.1.8 (May 26, 2004)"
+#define kLibraryVersion "@(#) LibNcFTP 3.2.4 (April 3, 2010)"
+
+/* This is used to verify validty of the data passed in. */
+#define kLibraryMagic "LibNcFTP 3.2.4"
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	/* Includes for Windows */
 #	pragma once
-#	pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
-#	pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
-#	pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
-#	pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
-#	pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
-#	pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
-#	pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
-#	pragma warning(disable : 4711)	// warning C4711: function selected for automatic inline expansion
+#	ifndef __MINGW32__
+#		pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
+#		pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
+#		pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
+#		pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
+#		pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
+#		pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
+#		pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
+#		pragma warning(disable : 4711)	// warning C4711: function selected for automatic inline expansion
+#	endif
+	/* We now try for at least Windows 2000 compatibility (0x0500).
+	 * The code will still work on older systems, though.
+	 * Prior versions used 0x0400 instead.
+	 */
 #	ifndef WINVER
-#		define WINVER 0x0400
+#		define WINVER 0x0500
 #	endif
 #	ifndef _WIN32_WINNT
-#		define _WIN32_WINNT 0x0400
+#		define _WIN32_WINNT 0x0500
 #	endif
 #	ifndef _INC_WINDOWS
 #		include <windows.h>	/* includes <winsock2.h> if _WIN32_WINNT >= 0x400 */
@@ -80,9 +89,8 @@ extern "C"
 #	ifndef _INC_SETJMP
 #		include <setjmp.h>
 #	endif
-#	define longest_int LONGLONG
-#	define longest_int LONGLONG
-#	define longest_uint ULONGLONG
+#	define longest_int __int64
+#	define longest_uint __uint64
 #	ifndef HAVE_LONG_LONG
 #		define HAVE_LONG_LONG 1
 #	endif
@@ -107,8 +115,12 @@ extern "C"
 #	ifndef main_void_return_t
 #		define main_void_return_t void
 #	endif
-#	define strcasecmp stricmp
-#	define strncasecmp strnicmp
+#	ifndef strcasecmp
+#		define strcasecmp _stricmp
+#		define strncasecmp _strnicmp
+#		define strdup _strdup
+#		define fdopen _fdopen
+#	endif
 #	ifndef mode_t
 #		define mode_t int
 #	endif
@@ -159,14 +171,6 @@ extern "C"
 #	endif
 #else
 	/* Includes for UNIX */
-#	if (defined(__linux__)) && (! defined(_LARGEFILE64_SOURCE)) && (! defined(NO_LARGEFILE64_SOURCE))
-		/* Define _LARGEFILE64_SOURCE or NO_LARGEFILE64_SOURCE to
-		 * quiet this warning.  Be sure to define this for ALL
-		 * code that uses LibNcFTP.
-		 */
-#		warning "<ncftp.h> is #defining _LARGEFILE64_SOURCE for you"
-#		define _LARGEFILE64_SOURCE 1
-#	endif
 #	include <unistd.h>
 #	include <sys/types.h>
 #	include <sys/stat.h>
@@ -201,12 +205,6 @@ extern "C"
 
 #include "ncftp_errno.h"
 
-/* This is used to verify validty of the data passed in.
- * It also specifies the minimum version that is binary-compatibile with
- * this version.  (So this may not necessarily be kLibraryVersion.)
- */
-#define kLibraryMagic "LibNcFTP 3.1.8"
-
 #ifndef longest_int
 #define longest_int long long
 #define longest_uint unsigned long long
@@ -273,10 +271,11 @@ typedef void (*FTPGetPassphraseProc)(const FTPCIPtr, FTPLineListPtr pwPrompt, ch
 typedef struct FTPConnectionInfo {
 	char magic[16];				/* Don't modify this field. */
 	
-	char host[64];				/* REQUIRED input parameter. */
-	char user[64];				/* OPTIONAL input parameter. */
-	char pass[64];				/* OPTIONAL input parameter. */
+	char host[128];				/* REQUIRED input parameter. */
+	char user[128];				/* OPTIONAL input parameter. */
+	char pass[256];				/* OPTIONAL input parameter. */
 	char acct[64];				/* OPTIONAL input parameter. */
+	unsigned int passIsEmpty;		/* OPTIONAL input parameter. */
 	unsigned int port;			/* OPTIONAL input parameter. */
 
 	int errNo;				/* You may modify this if you want. */
@@ -285,6 +284,7 @@ typedef struct FTPConnectionInfo {
 	int lastFTPCmdResultNum;		/* You may modify this if you want. */
 
 	FILE *debugLog;				/* OPTIONAL input parameter. */
+	int debugTimestamping;			/* OPTIONAL input parameter. */
 	FTPLogProc debugLogProc;		/* OPTIONAL input parameter. */
 
 	unsigned int xferTimeout;		/* OPTIONAL input parameter. */
@@ -357,6 +357,9 @@ typedef struct FTPConnectionInfo {
 	int hasPASV;				/* Do not modify this field. */
 	int hasSIZE;				/* Do not modify this field. */
 	int hasMDTM;				/* Do not modify this field. */
+	int hasMDTM_set;			/* Do not modify this field. */
+	int hasMFMT;				/* Do not modify this field. */
+	int hasMFF;				/* Do not modify this field. */
 	int hasREST;				/* Do not modify this field. */
 	int hasNLST_a;				/* Do not modify this field. */
 	int hasNLST_d;				/* Do not modify this field. */
@@ -374,6 +377,7 @@ typedef struct FTPConnectionInfo {
 	int hasSITE_SBUFSIZ;			/* Do not modify this field. */
 	int hasSITE_SBUFSZ;			/* Do not modify this field. */
 	int hasSITE_BUFSIZE;			/* Do not modify this field. */
+	int hasRETR_tar;			/* Do not modify this field. */
 	int mlsFeatures;			/* Do not modify this field. */
 	int STATfileParamWorks;			/* Do not modify this field. */
 	int NLSTfileParamWorks;			/* Do not modify this field. */
@@ -415,7 +419,20 @@ typedef struct FTPConnectionInfo {
 
 	int doNotGetStartingWorkingDirectory;	/* You may modify this. */
 	char textEOLN[4];			/* Set automatically per platform. */
+	int asciiTranslationMode;		/* You may modify this. */
 	const char *manualOverrideFeatures;	/* You may modify this. */
+
+	/* Only set the following field if all the following are true:
+	 *   Your client FTP program is running on a private network (e.g. 192.168.1.0); and
+	 *   You are accessing a server outside your network (e.g., on the Internet); and
+	 *   Your network's firewall does NOT fix private addresses in PORT for you; and
+	 *   You know what your external IP address is.
+	 */
+	struct sockaddr_in clientKnownExternalAddr;	
+
+	struct sockaddr_in preferredLocalAddr;	/* You may modify this. */
+	int maxNumberOfSuccessivePASVAttempts;	/* You may modify this. */
+
 #if USE_SIO
 	char srlBuf[768];
 	SReadlineInfo ctrlSrl;			/* Do not use or modify. */
@@ -502,7 +519,7 @@ typedef struct MLstItem{
  * a user-installed signal handler.
  */
 #define kDefaultXferTimeout		600
-#define kDefaultConnTimeout		30
+#define kDefaultConnTimeout		10
 #define kDefaultCtrlTimeout		135
 #define kDefaultAbortTimeout		10
 #else
@@ -518,7 +535,7 @@ typedef struct MLstItem{
 
 /* Suggested timeout values, in seconds, if you use timeouts. */
 #define kSuggestedDefaultXferTimeout	600
-#define kSuggestedDefaultConnTimeout	30
+#define kSuggestedDefaultConnTimeout	10
 #define kSuggestedDefaultCtrlTimeout	135	/* 2*MSL, + slop */ 
 #define kSuggestedAbortTimeout		10
 
@@ -527,6 +544,8 @@ typedef struct MLstItem{
 
 #define kDefaultDataPortMode		kFallBackToSendPortMode
 
+#define kDefaultMaxNumberOfSuccessivePASVAttempts	3
+
 #define kRedialStatusDialing		0
 #define kRedialStatusSleeping		1
 
@@ -557,7 +576,8 @@ typedef struct MLstItem{
 #define kTarYes				1
 #define kTarNo				0
 
-#define UNIMPLEMENTED_CMD(a)		((a == 500) || (a == 502) || (a == 504))
+#define FTP_UNIMPLEMENTED_CMD(a)	((a == 500) || (a == 502) || (a == 504))
+#define FTP_SYNTAX_ERROR_IN_PARAMETERS(a)	(a == 501)	
 
 /* Possible values returned by GetDateAndTime. */
 #define kSizeUnknown			((longest_int) (-1))
@@ -667,6 +687,14 @@ typedef int (*FTPConfirmResumeUploadProc)(
 #define kServerTypeIBMFTPCS		14
 #define kServerTypePyramid		15
 
+/* This specifies the algorithm to use (if any) for processing the
+ * text during ASCII transfers.
+ */
+#define kAsciiTranslationModeNone	(-1)
+#define kAsciiTranslationModeFixEOLNs	0
+#define kAsciiTranslationModeStripCRs	1
+#define kAsciiTranslationModeDefault	kAsciiTranslationModeFixEOLNs
+
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	/* Windows has separate functions to close and ioctl sockets. */
 #else
@@ -747,9 +775,9 @@ int WinStat64(const char *const path, struct WinStat64 *const stp);
 
 /* The following block may be changed by configure script */
 #ifndef Stat
-#define Stat stat64
-#define Lstat lstat64
-#define Fstat fstat64
+#define Stat stat
+#define Lstat lstat
+#define Fstat fstat
 #endif
 
 #define kFtwNoAutoGrowButContinue (-1)
@@ -782,7 +810,7 @@ typedef struct FtwInfo {
 	size_t numFiles;
 	size_t numLinks;
 	const char *rlinkto;	/* only valid during FTPFtw() */
-	int reserved;
+	size_t direntbufSize;
 	void *direntbuf;
 	void *cip;		/* only valid during FTPFtw() */
 	void *userdata;
@@ -815,10 +843,10 @@ typedef struct GetoptInfo {
 #endif
 
 extern const char gLibNcFTPVersion[];
-extern int gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings;
+extern const char *gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings;
 
 #if (defined(__APPLE_CC__)) && (__APPLE_CC__ < 10000)
-#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (a == 0)
+#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (const char *) (&a)
 #	ifndef UNUSED
 #		define UNUSED(a) a
 #	endif
@@ -833,7 +861,7 @@ extern int gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings;
 #	endif
 #	define LIBNCFTP_USE_VAR(a)
 #else
-#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (a == 0)
+#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (const char *) (&a)
 #	ifndef UNUSED
 #		define UNUSED(a) a
 #	endif
@@ -858,10 +886,12 @@ int FTPDelete(const FTPCIPtr cip, const char *const pattern, const int recurse,
 int FTPFileExists(const FTPCIPtr cip, const char *const file);
 int FTPFileModificationTime(const FTPCIPtr cip, const char *const file, time_t *const mdtm);
 int FTPFileSize(const FTPCIPtr cip, const char *const file, longest_int *const size, const int type);
+longest_int FTPLocalASCIIFileSize(const char *const fn, char *buf, const size_t bufsize);
 int FTPFileSizeAndModificationTime(const FTPCIPtr cip, const char *const file, longest_int *const size, const int type, time_t *const mdtm);
 int FTPFileType(const FTPCIPtr cip, const char *const file, int *const ftype);
 int FTPFtw(const FTPCIPtr cip, const FtwInfoPtr ftwip, const char *const path, FtwProc proc);
 int FTPGetCWD(const FTPCIPtr cip, char *const newCwd, const size_t newCwdSize);
+int FTPGetFileToMemory(const FTPCIPtr cip, const char *const file, char *memBuf, const size_t maxNumberOfBytesToWriteToMemBuf, size_t *const numberOfBytesWrittenToMemBuf, const longest_int startPoint, const int deleteflag);
 int FTPGetFiles3(const FTPCIPtr cip, const char *pattern1, const char *const dstdir1, const int recurse, int doGlob, const int xtype, const int resumeflag, int appendflag, const int deleteflag, const int tarflag, const FTPConfirmResumeDownloadProc resumeProc, int UNUSED(reserved));
 int FTPGetOneFile3(const FTPCIPtr cip, const char *const file, const char *const dstfile, const int xtype, const int fdtouse, const int resumeflag, const int appendflag, const int deleteflag, const FTPConfirmResumeDownloadProc resumeProc, int UNUSED(reserved));
 int FTPInitConnectionInfo(const FTPLIPtr lip, const FTPCIPtr cip, size_t bufsize);
@@ -874,9 +904,12 @@ int FTPListToMemory2(const FTPCIPtr cip, const char *const pattern, const FTPLin
 int FTPLocalGlob(FTPCIPtr cip, FTPLineListPtr fileList, const char *pattern, int doGlob);
 int FTPLoginHost(const FTPCIPtr cip);
 int FTPMkdir(const FTPCIPtr cip, const char *const newDir, const int recurse);
+int FTPMkdir2(const FTPCIPtr cip, const char *const newDir, const int recurse, const char *const curDir);
+int FTPMkParentDir(const FTPCIPtr cip, const char *const path, const int recurse, const char *const curDir);
 int FTPOpenHost(const FTPCIPtr cip);
 int FTPOpenHostNoLogin(const FTPCIPtr cip);
 void FTPPerror(const FTPCIPtr cip, const int err, const int eerr, const char *const s1, const char *const s2);
+int FTPPutFileFromMemory(const FTPCIPtr cip, const char *volatile dstfile, const char *volatile src, const size_t srcLen, const int appendflag);
 int FTPPutFiles3(const FTPCIPtr cip, const char *const pattern, const char *const dstdir1, const int recurse, const int doGlob, const int xtype, int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
 int FTPPutOneFile3(const FTPCIPtr cip, const char *const file, const char *const dstfile, const int xtype, const int fdtouse, const int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
 int FTPReadLoginConfigFile(FTPCIPtr cip, const char *const fn);
@@ -917,10 +950,11 @@ void InitWinsock(void);
 void DisposeWinsock(void);
 char *StrDup(const char *);
 char *FGets(char *, size_t, FILE *);
-void GetHomeDir(char *, size_t);
+void GetHomeDir(char *const dst, const size_t size);
+void GetTmpDir(char *const dst, const size_t size);
 void GetUsrName(char *, size_t);
 void Scramble(unsigned char *dst, size_t dsize, unsigned char *src, char *key);
-double Duration(struct timeval *t0);
+double FTPDuration(struct timeval *t0);
 struct tm *Gmtime(time_t t, struct tm *const tp);
 struct tm *Localtime(time_t t, struct tm *const tp);
 time_t GetUTCOffset(const int mon, const int mday);
@@ -931,6 +965,7 @@ char *GetPass(const char *const prompt, char *const pwbuf, const size_t pwbufsiz
 int FilenameExtensionIndicatesASCII(const char *const pathName, const char *const extnList);
 void StrRemoveTrailingSlashes(char *dst);
 int StrToBoolOrInt(const char *s);
+int FTPFixPrivateAddr(struct sockaddr_in *maybePrivateAddr, struct sockaddr_in *knownNonPrivateAddrToUseIfNeeded);
 #if defined(WIN32) || defined(_WINDOWS) || defined(__CYGWIN__)
 char *StrFindLocalPathDelim(const char *src);
 char *StrRFindLocalPathDelim(const char *src);
@@ -962,7 +997,6 @@ int FTPPutFiles(const FTPCIPtr cip, const char *const pattern, const char *const
 int FTPPutFiles2(const FTPCIPtr cip, const char *const pattern, const char *const dstdir, const int recurse, const int doGlob, const int xtype, const int appendflag, const char *const tmppfx, const char *const tmpsfx);
 int FTPPutOneFileAscii(const FTPCIPtr cip, const char *const file, const char *const dstfile);
 int FTPPutFilesAscii(const FTPCIPtr cip, const char *const pattern, const char *const dstdir, const int recurse, const int doGlob);
-int FTPPutFileFromMemory(const FTPCIPtr cip, const char *volatile dstfile, const char *volatile src, const size_t srcLen, const int appendflag);
 
 /* Everything else below are private routines, or stuff for testing */
 int FTPInitConnectionInfo2(const FTPLIPtr lip, const FTPCIPtr cip, char *const buf, size_t bufSize);
@@ -970,6 +1004,7 @@ int FTPSendPort(const FTPCIPtr cip, struct sockaddr_in *saddr);
 int FTPSendPassive(const FTPCIPtr cip, struct sockaddr_in *saddr, int *weird);
 int FTPSetStartOffset(const FTPCIPtr cip, longest_int restartPt);
 void FTPCloseControlConnection(const FTPCIPtr cip);
+int FTPSendCommandStr(const FTPCIPtr cip, char *const command, const size_t siz);
 int FTPSendCommand(const FTPCIPtr cip, const char *const cmdspec, va_list ap)
 #if (defined(__GNUC__)) && (__GNUC__ >= 2)
 __attribute__ ((format (printf, 2, 0)))
@@ -981,11 +1016,21 @@ void FTPManualOverrideFeatures(const FTPCIPtr cip);
 int FTPMListOneFile(const FTPCIPtr cip, const char *const file, const MLstItemPtr mlip);
 void FTPInitializeAnonPassword(const FTPLIPtr);
 void FTPInitIOTimer(const FTPCIPtr);
-int FTPStartDataCmd(const FTPCIPtr, int, int, longest_int, const char *,...)
+int FTPStartDataCmd(const FTPCIPtr, const int, const int, const longest_int, const char *const ,...)
 #if (defined(__GNUC__)) && (__GNUC__ >= 2)
 __attribute__ ((format (printf, 5, 6)))
 #endif
 ;
+int FTPStartDataCmd2(const FTPCIPtr cip, const int netMode, const int type, const longest_int startPoint, char *const cmdstr, const size_t cmdstrSize, const char *variableCommandSpec, ...)
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((format (printf, 7, 8)))
+#endif
+;
+int FTPStartDataCmd3(const FTPCIPtr cip, const int netMode, const int type, const longest_int startPoint, char *const cmdstr, const size_t cmdstrSize, const char *const variableCommandSpec, va_list ap)
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((format (printf, 7, 0)))
+#endif
+;
 void FTPStartIOTimer(const FTPCIPtr);
 void FTPStopIOTimer(const FTPCIPtr);
 void FTPUpdateIOTimer(const FTPCIPtr);
@@ -1004,13 +1049,13 @@ int WaitResponse(const FTPCIPtr, unsigned int);
 int FTPLocalRecursiveFileList(FTPCIPtr, FTPLineListPtr, FTPFileInfoListPtr);
 int FTPLocalRecursiveFileList2(FTPCIPtr cip, FTPLineListPtr fileList, FTPFileInfoListPtr files, int erelative);
 
-int FTPMkdir2(const FTPCIPtr cip, const char *const newDir, const int recurse, const char *const curDir);
-
 /* FTPFtw() is recommended since it is more reliable (read: thorough),
  * but these two work faster.
  */
 int FTPRemoteRecursiveFileList(FTPCIPtr, FTPLineListPtr, FTPFileInfoListPtr);
 int FTPRemoteRecursiveFileList1(FTPCIPtr, char *const, FTPFileInfoListPtr);
+int FTPRemoteFtwProc(const FtwInfoPtr ftwip);
+int FTPRemoteRecursiveFileList2(FTPCIPtr cip, char *const rdir, FTPFileInfoListPtr files);
 
 int FTPRebuildConnectionInfo(const FTPLIPtr lip, const FTPCIPtr cip);
 
@@ -1020,6 +1065,7 @@ int FTPFileExistsNlst(const FTPCIPtr cip, const char *const file);
 int FTPFileExists2(const FTPCIPtr cip, const char *const file, const int tryMDTM, const int trySIZE, const int tryMLST, const int trySTAT, const int tryNLST);
 
 void FTPGetDateStr(time_t t, const char *fmt, char *const ltstr1, const size_t ltstr1size, char *const gtstr1, const size_t gtstr1size);
+int BindToEphemeralPortNumber(const int sockfd, struct sockaddr_in *const addrp, const int ephemLo, const int ephemHi);
 int BufferGets(char *, size_t, int, char *, char **, char **, size_t);
 void DisposeFileInfoListContents(FTPFileInfoListPtr);
 void InitFileInfoList(FTPFileInfoListPtr);
